@@ -38,12 +38,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
@@ -57,20 +57,33 @@
 #undef BI_PNG
 #define BI_PNG  5
 #if !defined(MSWINDOWS) || defined(__MINGW32__)
+#undef BI_RGB
 #define BI_RGB  0
+#undef BI_RLE8
 #define BI_RLE8  1
+#undef BI_RLE4
 #define BI_RLE4  2
+#undef BI_BITFIELDS
 #define BI_BITFIELDS  3
 
+#undef LCS_CALIBRATED_RBG
 #define LCS_CALIBRATED_RBG  0
+#undef LCS_sRGB
 #define LCS_sRGB  1
+#undef LCS_WINDOWS_COLOR_SPACE
 #define LCS_WINDOWS_COLOR_SPACE  2
+#undef PROFILE_LINKED
 #define PROFILE_LINKED  3
+#undef PROFILE_EMBEDDED
 #define PROFILE_EMBEDDED  4
 
+#undef LCS_GM_BUSINESS
 #define LCS_GM_BUSINESS  1  /* Saturation */
+#undef LCS_GM_GRAPHICS
 #define LCS_GM_GRAPHICS  2  /* Relative */
+#undef LCS_GM_IMAGES
 #define LCS_GM_IMAGES  4  /* Perceptual */
+#undef LCS_GM_ABS_COLORIMETRIC
 #define LCS_GM_ABS_COLORIMETRIC  8  /* Absolute */
 #endif
 
@@ -279,7 +292,8 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
       }
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                  LoadImageText,image->filename))
+                                  LoadImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   (void) ReadBlobByte(image);  /* end of line */
@@ -369,7 +383,8 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     *q++=0x00;
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                  SaveImageText,image->filename))
+                                  SaveImageText,image->filename,
+				  image->columns,image->rows))
         break;
   }
   /*
@@ -762,17 +777,20 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
             }
             profile_data=ReadBlobLSBLong(image);
-            profile_data=profile_data;
             profile_size=ReadBlobLSBLong(image);
-            profile_size=profile_size;
+	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				  "  Profile: size %lu data %lu",
+				  profile_size,profile_data);
             (void) ReadBlobLSBLong(image);  /* Reserved byte */
           }
       }
 
-    if ((bmp_info.compression != BI_RGB) &&
-        ((magick_off_t) bmp_info.file_size != GetBlobSize(image)))
+    if ((magick_off_t) bmp_info.file_size > GetBlobSize(image))
       ThrowReaderException(CorruptImageError,LengthAndFilesizeDoNotMatch,
         image);
+    if (logging && (magick_off_t) bmp_info.file_size < GetBlobSize(image))
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+         "Discarding all data beyond bmp_info.file_size");
     if (bmp_info.width <= 0)
       ThrowReaderException(CorruptImageWarning,NegativeOrZeroImageSize,image);
     if (bmp_info.height == 0)
@@ -986,7 +1004,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1014,7 +1033,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1045,7 +1065,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1103,7 +1124,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1136,7 +1158,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1195,7 +1218,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 status=MagickMonitorFormatted(image->rows-y-1,image->rows,
                                               exception,LoadImageText,
-                                              image->filename);
+                                              image->filename,
+					      image->columns,image->rows);
                 if (status == False)
                   break;
               }
@@ -1420,8 +1444,8 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
     scene,
     type;
 
-  const unsigned char
-    *color_profile=0;
+/*   const unsigned char */
+/*     *color_profile=0; */
 
   size_t
     color_profile_length=0;
@@ -1447,8 +1471,9 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
 
   /*
     Retrieve color profile from Image (if any)
+    FIXME: is color profile support writing not properly implemented?
   */
-  color_profile=GetImageProfile(image,"ICM",&color_profile_length);
+  /* color_profile= */ (void) GetImageProfile(image,"ICM",&color_profile_length);
 
   do
   {
@@ -1582,7 +1607,8 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                              SaveImageText,image->filename))
+                                              SaveImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
           break;
@@ -1612,7 +1638,8 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
          if (image->previous == (Image *) NULL)
            if (QuantumTick(y,image->rows))
              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                         SaveImageText,image->filename))
+                                         SaveImageText,image->filename,
+					 image->columns,image->rows))
                break;
         }
         break;
@@ -1641,7 +1668,8 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         break;
@@ -1676,7 +1704,8 @@ static unsigned int WriteBMPImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+					  image->columns,image->rows))
                 break;
         }
         break;
