@@ -37,6 +37,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/analyze.h"
 #include "magick/blob.h"
 #include "magick/color.h"
 #include "magick/colormap.h"
@@ -440,7 +441,6 @@ static Image *ReadMATImage(const ImageInfo *image_info, ExceptionInfo *exception
   long ldblk;
   unsigned char *BImgBuff = NULL;
   double MinVal, MaxVal;
-  magick_uint32_t Unknown6;
   unsigned z;
   int logging;
   int sample_size;
@@ -554,7 +554,7 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     {     
       case  8: z=1; break;			/* 2D matrix*/
       case 12: z = ReadBlobXXXLong(image2);	/* 3D matrix RGB*/
-  	       Unknown6 = ReadBlobXXXLong(image2);
+	       (void) ReadBlobXXXLong(image2);  /* Unknown6 */
 	       if(z!=3) ThrowReaderException(CoderError, MultidimensionalMatricesAreNotSupported,
                          image);
 	       break;
@@ -944,7 +944,6 @@ static unsigned int WriteMATLABImage(const ImageInfo *image_info,Image *image)
 {
   long y;
   unsigned z;
-  const PixelPacket *q;
   unsigned int status;
   int logging;
   unsigned long DataSize;
@@ -1042,7 +1041,7 @@ static unsigned int WriteMATLABImage(const ImageInfo *image_info,Image *image)
         for (y=0; y<(long)image->columns; y++)
 	{
           progress_quantum++;
-          q = AcquireImagePixels(image,y,0,1,image->rows,&image->exception);
+          (void) AcquireImagePixels(image,y,0,1,image->rows,&image->exception);
           (void) ExportImagePixelArea(image,z2qtype[z],8,pixels,0,0);
           (void) WriteBlob(image,image->rows,pixels);
           if (QuantumTick(progress_quantum,progress_span))
@@ -1105,7 +1104,12 @@ ModuleExport void RegisterMATImage(void)
   entry->decoder = (DecoderHandler) ReadMATImage;
   entry->encoder = (EncoderHandler) WriteMATLABImage;
   entry->seekable_stream = True;
-  entry->description = "MATLAB image format";
+  entry->description =
+#if defined(HasZLIB)
+                        "MATLAB Level 5.0-7.0 image formats";
+#else
+			"MATLAB Level 5.0-6.0 image formats";
+#endif
   entry->module = "MAT";
   entry->blob_support=False;
   (void) RegisterMagickInfo(entry);
