@@ -21,7 +21,7 @@
 MagickExport const char *BlobModeToString(BlobMode blob_mode)
 {
   const char
-    *mode_string="Undefined";
+    *mode_string="?";
 
   switch (blob_mode)
   {
@@ -50,11 +50,12 @@ MagickExport const char *BlobModeToString(BlobMode blob_mode)
 MagickExport const char *ChannelTypeToString(const ChannelType channel)
 {
   const char
-    *channel_type="undefined";
+    *channel_type="?";
 
   switch (channel)
     {
     case UndefinedChannel:
+      channel_type="undefined";
       break;
     case RedChannel:
       channel_type="red";
@@ -132,7 +133,7 @@ MagickExport ChannelType StringToChannelType(const char *option)
 MagickExport const char* ClassTypeToString(const ClassType class_type)
 {
   const char
-    *log_class_type="Unknown";
+    *log_class_type="?";
 
   switch (class_type)
     {
@@ -155,7 +156,7 @@ MagickExport const char* ClassTypeToString(const ClassType class_type)
 MagickExport const char *ColorspaceTypeToString(const ColorspaceType colorspace)
 {
   const char
-    *colorspace_type = "Unknown";
+    *colorspace_type = "?";
   
   switch (colorspace)
     {
@@ -277,7 +278,7 @@ MagickExport ColorspaceType StringToColorspaceType(const char *option)
 MagickExport const char *CompositeOperatorToString(const CompositeOperator composite_op)
 {
   const char
-    *composite_op_text = "Undefined";
+    *composite_op_text = "?";
 
   switch (composite_op)
     {
@@ -486,7 +487,7 @@ MagickExport CompositeOperator StringToCompositeOperator(const char *option)
 MagickExport const char* CompressionTypeToString(const CompressionType compression_type)
 {
   const char
-    *compression_string="Unknown";
+    *compression_string="?";
 
   switch (compression_type)
     {
@@ -520,6 +521,18 @@ MagickExport const char* CompressionTypeToString(const CompressionType compressi
     case ZipCompression:
       compression_string="Zip";
       break;
+    case LZMACompression:
+      compression_string="LZMA";
+      break;
+    case JPEG2000Compression:
+      compression_string="JPEG2000";
+      break;
+    case JBIG1Compression:
+      compression_string="JBIG1";
+      break;
+    case JBIG2Compression:
+      compression_string="JBIG2";
+      break;
     }
   return compression_string;
 }
@@ -538,6 +551,8 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=BZipCompression;
   else if (LocaleCompare("Fax",option) == 0)
     compression_type=FaxCompression;
+  else if (LocaleCompare("Group3",option) == 0)
+    compression_type=Group3Compression;
   else if (LocaleCompare("Group4",option) == 0)
     compression_type=Group4Compression;
   else if (LocaleCompare("JPEG",option) == 0)
@@ -552,6 +567,16 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=ZipCompression;
   else if (LocaleCompare("GZip",option) == 0)
     compression_type=ZipCompression;
+  else if ((LocaleCompare("LZMA",option) == 0) ||
+	   (LocaleCompare("LZMA2",option) == 0))
+    compression_type=LZMACompression;
+  else if (LocaleCompare("JPEG2000",option) == 0)
+    compression_type=JPEG2000Compression;
+  else if ((LocaleCompare("JBIG",option) == 0) ||
+	   (LocaleCompare("JBIG1",option) == 0))
+    compression_type=JBIG1Compression;
+  else if (LocaleCompare("JBIG2",option) == 0)
+    compression_type=JBIG2Compression;
 
   return compression_type;
 }
@@ -562,7 +587,7 @@ MagickExport CompressionType StringToCompressionType(const char *option)
 MagickExport const char *ConfirmAccessModeToString(const ConfirmAccessMode access_mode)
 {
   const char
-    *mode_string="Undefined";
+    *mode_string="?";
 
   switch(access_mode)
     {
@@ -598,7 +623,7 @@ MagickExport const char *ConfirmAccessModeToString(const ConfirmAccessMode acces
 MagickExport const char *EndianTypeToString(const EndianType endian_type)
 {
   const char
-    *endian_string="Undefined";
+    *endian_string="?";
 
   switch (endian_type)
     {
@@ -722,11 +747,12 @@ MagickExport const char *
 HighlightStyleToString(const HighlightStyle difference_style)
 {
   const char
-    *style="Undefined";;
+    *style="?";
 
   switch(difference_style)
     {
     case UndefinedHighlightStyle:
+      style="Undefined";
       break;
     case AssignHighlightStyle:
       style="Assign";
@@ -767,7 +793,7 @@ StringToHighlightStyle(const char *option)
 MagickExport const char *ImageTypeToString(const ImageType image_type)
 {
   const char
-    *p = "Unknown";
+    *p = "?";
 
   switch (image_type)
     {
@@ -843,7 +869,7 @@ MagickExport ImageType StringToImageType(const char *option)
 MagickExport const char *InterlaceTypeToString(const InterlaceType interlace_type)
 {
   const char
-    *interlace_string = "Undefined";
+    *interlace_string = "?";
 
   switch (interlace_type)
     {
@@ -889,7 +915,7 @@ MagickExport InterlaceType StringToInterlaceType(const char *option)
 MagickExport const char *MapModeToString(MapMode map_mode)
 {
   char
-    *mode_string="Undefined";
+    *mode_string="?";
 
   switch (map_mode)
   {
@@ -968,7 +994,7 @@ MagickExport MetricType StringToMetricType(const char *option)
 MagickExport const char *OrientationTypeToString(const OrientationType orientation_type)
 {
   const char *
-    orientation = "Unknown";
+    orientation = "?";
 
   switch (orientation_type)
     {
@@ -1153,11 +1179,12 @@ MagickExport QuantumOperator StringToQuantumOperator(const char *option)
 MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_operator)
 {
   const char
-    *operator_text = "undefined";
+    *operator_text = "?";
 
   switch (quantum_operator)
     {
     case UndefinedQuantumOp:
+      operator_text="undefined";
       break;
     case AddQuantumOp:
       operator_text="add";
@@ -1248,7 +1275,7 @@ MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_o
 MagickExport const char *ResizeFilterToString(const FilterTypes filter)
 {
   const char *
-    filter_string = "Unknown";
+    filter_string = "?";
 
   switch (filter)
     {
@@ -1325,6 +1352,8 @@ MagickExport ResourceType StringToResourceType(const char *option)
     resource_type=MemoryResource;
   else if (LocaleCompare("Pixels",option) == 0)
     resource_type=PixelsResource;
+  else if (LocaleCompare("Threads",option) == 0)
+    resource_type=ThreadsResource;
   return resource_type;
 }
 
@@ -1334,7 +1363,7 @@ MagickExport ResourceType StringToResourceType(const char *option)
 MagickExport const char *StorageTypeToString(const StorageType storage_type)
 {
     const char
-    *p = "Unknown";
+    *p = "?";
 
     switch (storage_type)
       {
@@ -1367,7 +1396,7 @@ MagickExport const char *StorageTypeToString(const StorageType storage_type)
 MagickExport const char *QuantumSampleTypeToString(const QuantumSampleType sample_type)
 {
     const char
-    *p = "Unknown";
+    *p = "?";
 
     switch (sample_type)
       {
@@ -1429,7 +1458,7 @@ MagickExport const char *StyleTypeToString(StyleType style)
 MagickExport const char *QuantumTypeToString(const QuantumType quantum_type)
 {
   const char
-    *p = "Unknown";
+    *p = "?";
 
   switch (quantum_type)
     {
