@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2009 GraphicsMagick Group
+% Copyright (C) 2003-2012 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -348,6 +348,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
 MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
 {
   double
+    epsilon,
     normalize,
     value;
 
@@ -359,6 +360,9 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
 
   if (radius > 0.0)
     return((int) (2.0*ceil(radius)+1.0));
+  epsilon=1.0/MaxRGBDouble;
+  if (epsilon < MagickEpsilon)
+    epsilon=MagickEpsilon;
   for (width=5; ;)
   {
     normalize=0.0;
@@ -366,7 +370,7 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
       normalize+=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma);
     u=width/2;
     value=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGBDouble*value) <= 0)
+    if (value < epsilon)
       break;
     width+=2;
   }
@@ -377,6 +381,7 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
 {
   double
     alpha,
+    epsilon,
     normalize,
     value;
 
@@ -389,6 +394,9 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
 
   if (radius > 0.0)
     return((int) (2.0*ceil(radius)+1.0));
+  epsilon=1.0/MaxRGBDouble;
+  if (epsilon < MagickEpsilon)
+    epsilon=MagickEpsilon;
   for (width=5; ;)
   {
     normalize=0.0;
@@ -402,7 +410,7 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
     }
     v=width/2;
     value=exp(-((double) v*v)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGBDouble*value) <= 0)
+    if (value < epsilon)
       break;
     width+=2;
   }
@@ -624,18 +632,21 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
 %
 %
 */
+
 MagickExport void Hull(const long x_offset,const long y_offset,
-  const unsigned long columns,const unsigned long rows,Quantum *f,Quantum *g,
-  const int polarity)
+		       const unsigned long columns,const unsigned long rows,Quantum *f,Quantum *g,
+		       const int polarity)
 {
-  double
-    v,
+#if QuantumDepth > 16
+  typedef double SignedQuantum;
+#else
+  typedef int SignedQuantum;
+#endif
+
+  long
     y;
 
-  register long
-    x;
-
-  register Quantum
+  Quantum
     *p,
     *q,
     *r,
@@ -646,76 +657,102 @@ MagickExport void Hull(const long x_offset,const long y_offset,
   p=f+(columns+2);
   q=g+(columns+2);
   r=p+(y_offset*((long) columns+2)+x_offset);
+#if defined(HAVE_OPENMP)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime)
+#  else
+#    if defined(USE_STATIC_SCHEDULING_ONLY)
+#      pragma omp parallel for schedule(static)
+#    else
+#      pragma omp parallel for schedule(guided)
+#    endif
+#  endif
+#endif
   for (y=0; y < (long) rows; y++)
-  {
-    p++;
-    q++;
-    r++;
-    if (polarity > 0)
-      for (x=(long) columns; x > 0; x--)
-      {
-        v=(*p);
-        if (*r >= (v+ScaleCharToQuantum(2)))
-          v+=ScaleCharToQuantum(1);
-        *q=(Quantum) v;
-        p++;
-        q++;
-        r++;
-      }
-    else
-      for (x=(long) columns; x > 0; x--)
-      {
-        v=(*p);
-        if (*r <= (v-(long) ScaleCharToQuantum(2)))
-          v-=(long) ScaleCharToQuantum(1);
-        *q=(Quantum) v;
-        p++;
-        q++;
-        r++;
-      }
-    p++;
-    q++;
-    r++;
-  }
+    {
+      SignedQuantum
+	v;
+
+      unsigned long
+	x;
+
+      unsigned int
+	index;
+
+      index=(2*y+1)+y*columns;
+      if (polarity > 0)
+	{
+	  for (x=columns ; x != 0; x--)
+	    {
+	      v=(p[index]);
+	      if (r[index] >= (v+ScaleCharToQuantum(2)))
+		v+=ScaleCharToQuantum(1);
+	      q[index]=(Quantum) v;
+	      index++;
+	    }
+	}
+      else
+	{
+	  for (x=columns ; x != 0; x--)
+	    {
+	      v=(p[index]);
+	      if (r[index] <= (v-(long) ScaleCharToQuantum(2)))
+		v-=(long) ScaleCharToQuantum(1);
+	      q[index]=(Quantum) v;
+	      index++;
+	    }
+	}
+    }
   p=f+(columns+2);
   q=g+(columns+2);
   r=q+(y_offset*((long) columns+2)+x_offset);
   s=q-(y_offset*((long) columns+2)+x_offset);
+#if defined(HAVE_OPENMP)
+#  if defined(TUNE_OPENMP)
+#    pragma omp parallel for schedule(runtime)
+#  else
+#    if defined(USE_STATIC_SCHEDULING_ONLY)
+#      pragma omp parallel for schedule(static)
+#    else
+#      pragma omp parallel for schedule(guided)
+#    endif
+#  endif
+#endif
   for (y=0; y < (long) rows; y++)
-  {
-    p++;
-    q++;
-    r++;
-    s++;
-    if (polarity > 0)
-      for (x=(long) columns; x > 0; x--)
-      {
-        v=(*q);
-        if ((*s >= (v+ScaleCharToQuantum(2))) && (*r > v))
-          v+=ScaleCharToQuantum(1);
-        *p=(Quantum) v;
-        p++;
-        q++;
-        r++;
-        s++;
-      }
-    else
-      for (x=(long) columns; x > 0; x--)
-      {
-        v=(*q);
-        if ((*s <= (v-(long) ScaleCharToQuantum(2))) && (*r < v))
-          v-=(long) ScaleCharToQuantum(1);
-        *p=(Quantum) v;
-        p++;
-        q++;
-        r++;
-        s++;
-      }
-    p++;
-    q++;
-    r++;
-    s++;
-  }
+    {
+      SignedQuantum
+	v;
+
+      unsigned long
+	x;
+
+      unsigned int
+	index;
+
+      index=(2*y+1)+y*columns;
+      if (polarity > 0)
+	{
+	  for (x=columns ; x != 0; x--)
+	    {
+	      v=(q[index]);
+	      if ((s[index] >= (v+ScaleCharToQuantum(2))) && (r[index] > v))
+		v+=ScaleCharToQuantum(1);
+	      p[index]=(Quantum) v;
+	      index++;
+	    }
+	}
+      else
+	{
+	  for (x=columns ; x != 0; x--)
+	    {
+	      v=(q[index]);
+	      if ((s[index] <= (v-(long) ScaleCharToQuantum(2))) && (r[index] < v))
+		v-=(long) ScaleCharToQuantum(1);
+	      p[index]=(Quantum) v;
+	      index++;
+	    }
+	}
+    }
 }
 
 /*
